<?php
/*
Plugin Name: Category Icons
Plugin URI: http://wordpress.org/extend/plugins/category-icons/
Description: Assign icons to your categories easily. Thanks to the following contributors : Kristian Bollnow, Hugo Chen, Kalin Dimitrov, Gianni Diurno, Samuel Kroslak, Jean-Christophe Marie, Andrew Senyshyn, Henrik Schack, Vincent Sparreboom, TechnopodMan, Oliver Weichhold.
Version: 2.0.1
Author: Brahim Machkouri
Author URI: http://categoryicons.wordpress.com/
Compatibility : WordPress 2.3 up to 2.5.1

----------------------------------------------------------------------------

	Original version (1.5) : Ivan Georgiev
	
	Thanks to the following contributors :
	
	Kristian Bollnow (German translation), 
	Hugo Chen (Chinese Taiwan translation),
	Kalin Dimitrov (Bulgarian translation),
	Gianni Diurno (Italian translation),
	Samuel Kroslak (Slovak translation), 
	Jean-Christophe Marie (Bug correction),
	Andrew Senyshyn (Ukrainian translation), 
	Henrik Schack (Danish translation),
	Vincent Sparreboom (Dutch translation), 
	TechnopodMan (Spanish translation).
	Oliver Weichhold (Priority feature), 
	
----------------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Initialization and Hooks
$wpdb->ig_caticons = $table_prefix.'ig_caticons';
add_action('init', 'ig_caticons_install');
add_action('admin_menu', 'ig_caticons_adminmenu');
add_action('admin_menu', 'bm_caticons_add_js_libs');
add_action( 'admin_notices', 'bm_caticons_update_nag', 3 );
add_filter('pre_kses', 'bm_caticons_plugin_description' );

/**
 * If a RSS feed is created, 'raise the flag'
 */
function bm_caticons_rss_flag() {
	global $bm_caticons_rss;
	$bm_caticons_rss = true;
}
add_action('rss2_head','bm_caticons_rss_flag');
//add_action('rss_head','bm_caticons_rss_flag'); // Not working with RSS 0.92
add_action('rdf_header','bm_caticons_rss_flag'); // only for Safari
add_action('atom_ns','bm_caticons_rss_flag');

/**
 * Inject the icons into the feeds
 * Working only with RSS2 and Atom. (And RDF, but only in Safari)
 * @param string The content to process
 * @return string The feed content
 */
function bm_caticons_rss($content) {
	global $bm_caticons_rss;
	if ( $bm_caticons_rss && 1 == get_option('igcaticons_rssfeeds')  ) // If the rss flag is raised, inject icons
		$content = get_cat_icon('echo=false&link=false').'<br/>'.$content;
	return $content;
}
add_filter('the_excerpt_rss','bm_caticons_rss');
add_filter('the_content','bm_caticons_rss'); 
add_filter('the_content_rss','bm_caticons_rss'); 

/**
 * WordPress Template Tag to Insert Category Icon
 * @param int $fit_width Maximum width of the image, or desired width if expand is true. Default : -1
 * @param int $fit_height Maximum height (or desired height if $expanded=true) of the image. Default : -1
 * @param boolean $expand Whether the image should be expanded to fit the rectangle specified by fit_xxx. Default : false
 * @param int $cat Category ID. If not specified, the current category is used or the current post's category. Default : 
 * @param boolean $small Use the small icon. Default : false
 * @param string $prefix String to echo before the image tag. If no image, no output. Default : 
 * @param string $suffix String to echo after the image tag. Ignored if no image found. Default : 
 * @param string $class Class attribute for the image tag. Default : 
 * @param boolean $link If true the image is made a hyperlink (wrapped by anchor tag). Default : true
 * @param boolean $echo If true the html code will be 'echoed'. If no, it'll be returned. Default : true
 * @param boolean $use_priority If true, only the most prioritized icon will be displayed. Default : false
 * @param int $max_icons Maximum number of icons to display. Default : 3
 * @param boolean $vertical_display If true, displays the icons vertically. Default : false
 * @param boolean $border If true, displays the icon with a border. If false, no border is diplayed.
 * @return boolean True if image found.
 */
function get_cat_icon($params='') {
	parse_str($params, $p);
	if (!isset($p['fit_width'])) $p['fit_width']=get_option('igcaticons_width');//-1;
	if (!isset($p['fit_height'])) $p['fit_height']=get_option('igcaticons_height');//-1;
	if (!isset($p['expand'])) $p['expand']=false;
	//if (!isset($p['cat'])) $p['cat']=$GLOBALS['cat'];
	if (!isset($p['small'])) $p['small']=get_option('igcaticons_use_small');//false;
	if (!isset($p['prefix'])) $p['prefix'] = '';
	if (!isset($p['suffix'])) $p['suffix'] = '';
	if (!isset($p['class'])) $p['class'] = '';
	if (!isset($p['link'])) $p['link'] = true;
	if (!isset($p['echo'])) $p['echo']=true; 
	if (!isset($p['use_priority'])) $p['use_priority'] = false;
	if (!isset($p['max_icons'])) $p['max_icons']=get_option('igcaticons_max_icons'); 
	if (!isset($p['vertical_display'])) $p['vertical_display']=false;
	//if (!isset($p['border'])) $p['border']=''; 
	if (!isset($p['align'])) $p['align']=false; 

	$p['vertical_display'] = bm_caticons_getbool($p['vertical_display']);
	$p['expand'] = bm_caticons_getbool($p['expand']);
	$p['small'] = bm_caticons_getbool($p['small']);
	$p['echo'] = bm_caticons_getbool($p['echo']);
	if (isset($p['border'])) $p['border'] = bm_caticons_getbool($p['border']);
	$p['use_priority'] = bm_caticons_getbool($p['use_priority']);
	$p['link'] = bm_caticons_getbool($p['link']);
	stripslaghes_gpc_arr($p);
	if (!isset($p['cat']) && isset($GLOBALS['post'])) {
		$catlist = get_the_category($GLOBALS['post']->ID);
		if (is_array($catlist)) {
			$p['cat'] = array();
			foreach ($catlist as $categorie) {
				$p['cat'][] =  $categorie->cat_ID;
			}
		}
	}
	
	if (!isset($p['cat'][0])) 
		return;
	
	if (!is_array($p['cat'])) {
		$categorie=$p['cat'];
		$p['cat'] = array();
		$p['cat'][] = $categorie;
	}
	
	$nb_icons = 0;
	$boxv_width = 0;
	$urlbegin = '';
	$urlend = '';
	
	for ($i=0; $i<count($p['cat']); $i++)	{
		
		if($p['use_priority']) // if you decide to use the priority feature
			list( $p['cat'][$i] , $priority , $icon , $small_icon ) = ow_caticons_get_priority_icon( $p['cat'] );
		else 
			list( $priority , $icon , $small_icon ) = ig_caticons_get_icons( $p['cat'][$i]);
		
		if ($p['small']) {// If we choose to display the small icon
			$file = ig_caticons_path().'/'.$small_icon;
			$url = ig_caticons_url().'/'.$small_icon;
			if (!is_file($file)) {// if the small icon can't be found, the normal one will be loaded
				$file = ig_caticons_path().'/'.$icon;
				$url = ig_caticons_url().'/'.$icon;
			}
		} else {
			$file = ig_caticons_path().'/'.$icon;
			$url = ig_caticons_url().'/'.$icon;
			if ( !is_file($file) ) {// If the normal icon can't be found, the small one will be loaded
				$file = ig_caticons_path().'/'.$small_icon;
				$url = ig_caticons_url().'/'.$small_icon;
			}
		}
		
		if ( is_file($file) ) {
		
			if ( $p['link'] ) {
				$urlbegin = '<a href="'.get_category_link($p['cat'][$i]).'" title="'.get_cat_name($p['cat'][$i]).'">';
				$urlend = '</a>';
			}
		
			list( $width , $height , $type , $attr) = getimagesize($file);
			$w = $width;
			$h = $height;
			
			if (!empty($p['fit_width']) || !empty($p['fit_height']))
				list($w, $h) = ig_fit_rect($width, $height, $p['fit_width'], $p['fit_height'], $p['expand']);
			
			if ($p['vertical_display'] && $w > $boxv_width) 
				$boxv_width = $w; 
			
			$cat_icons .= $p['prefix'].$urlbegin.'<img ';
			
			if (!empty($p['class'])) 
				$cat_icons .= 'class="'.$p['class'].'" ';
				
			
			if (isset($p['border'])) {
				if ($p['border']) 
					$cat_icons .= 'border="1" ';
				else
					$cat_icons .= 'border="0" ';
			}
			
			if (!$p['align']===false) 
				$cat_icons .= 'align="'.$p['align'].'" ';
						
			$cat_icons .= 'src="'.$url.'" width="'.$w.'" height="'.$h.'" alt=" " title="'.get_cat_name($p['cat'][$i]).'" />'.$urlend.$p['suffix'];
			$nb_icons++;
		}
		
		if ( $p['use_priority'] ) 
			break;
			
		if ( $nb_icons == $p['max_icons']) 
			break;
	}
	
	if ( $p['vertical_display'] && 1 < count($p['cat']) ) 
		$cat_icons = '<div class="caticons" style="text-align:center;float:left;width: '.$boxv_width.'px;">'.$cat_icons.'</div>';
	
	if ($p['echo'])	{
		if (empty($cat_icons)) 	
			return false;
		else {
			echo $cat_icons;
			return true;
		}
	} 
	else 
		return $cat_icons;
}

/**
 * WordPress Template Tag to Inject the html image source code of each category contained in the list given as parameter
 * @author Brahim MACHKOURI
 * @param string $list HTML code of the category list 
 * @param string $pamameters The parameters of get_cat_icon() function 
 */
function put_cat_icons($list,$parameters='',$type='category') {
	global $wpdb;
	
	$request = "SELECT term_id
				FROM $wpdb->term_relationships
				right join $wpdb->term_taxonomy on $wpdb->term_relationships.term_taxonomy_id = $wpdb->term_taxonomy.term_taxonomy_id
				left join $wpdb->posts on $wpdb->term_relationships.object_id = $wpdb->posts.ID
				where post_type = 'page' and $wpdb->term_taxonomy.taxonomy = 'category' and ID=";
	$nb_max_icons_page = get_option('igcaticons_max_icons');
	$cats = array();
	
	if ('category' == $type) $type = 'cat';
	if ($type != 'cat' && $type != 'page') $type = 'cat';
		
	if (empty($parameters)) // If no parameter is given, the ones from the options will be taken
		$parameters = 'fit_width='.get_option('igcaticons_width').'&fit_height='.get_option('igcaticons_height').'&small='.(bm_caticons_getbool(get_option('igcaticons_use_small')) ?'true':'false');
	else {
		parse_str($parameters, $p);
		if ( !isset($p['icons_only']) ) 
			$p['icons_only']=0;
			
		if ( isset($p['icons_only']) && 'true' == strtolower( $p['icons_only'] ) ) 
			$p['icons_only']=1;
	}
	
	if (!empty($list)) {
		$myarray = bm_caticons_url_extractor($list);
		foreach ($myarray as $child) {
			$cats = array();
			$pos = strpos($child[1],$type.'=');
			if ($pos>0)	{ // standard permalinks 
				parse_str(substr($child[1],$pos, strlen($child[1])-$pos));
				if (isset($page) && $page >0) { // get the category from page id
					$cats = array();
					$cats = $wpdb->get_results($wpdb->prepare($request.$page),ARRAY_A);
				}
				else {
					$cats[] = $cat;
				}
			}
			else {// permalinks not standard
				$cats = array();
				$temp = explode('/',($child[1]));
				$name = '';
				do 
					$name = array_pop($temp); // get the last non empty/null string from the array
				while (is_null($name) || empty($name));
				if ($type == 'cat') {
					$cat  = $wpdb->get_var($wpdb->prepare("SELECT term_id FROM `$wpdb->terms` WHERE slug = '$name'"),0,0);
					$cats[] = $cat;
				}
				else {
					$cats = array();
					$page  = $wpdb->get_var($wpdb->prepare("SELECT ID FROM `$wpdb->posts` WHERE post_name = '$name'"),0,0);
					$cats = $wpdb->get_results($wpdb->prepare($request.$page),ARRAY_A);
				}
			}
			if (is_array($cats[0])) { // Pages Icons
				$temp = array();
				for ($i=0; $i<count($cats);$i++) {
					$temp[$i]= $cats[$i]['term_id'];
				}
				$cats = $temp;
				unset($temp);
			}
			$img = '';
			$i=1;
			if (count($cats)>0) {
				foreach ($cats as $categ) {
					
					if ($categ == 1 && $type == 'page') continue;// Not display uncategorized icon for a page
					if ($i > $nb_max_icons_page) break;
					$image = get_cat_icon($parameters.'&cat='.$categ.'&echo=false&link=false');
					//if (!isset($p['class'])) $class = 'caticonspacer';
					 $class = 'caticonspacer';
					if ( !empty($image)  && 'image' == get_option('igcaticons_spacerstype')) {
						$spacer_left_image = '<img class="'.$p['class'].'" alt="'.get_cat_name($categ).'" title="'.get_cat_name($categ).'" width="'.get_option('igcaticons_sidebar_leftspacer_width').'" height="'.get_option('igcaticons_sidebar_leftspacer_height').'" src="'.trailingslashit(get_option('siteurl')).PLUGINDIR.'/category-icons/images/spacer.gif" />';
						$spacer_right_image = '<img class="'.$p['class'].'" alt="'.get_cat_name($categ).'" title="'.get_cat_name($categ).'" width="'.get_option('igcaticons_sidebar_rightspacer_width').'" height="'.get_option('igcaticons_sidebar_rightspacer_height').'" src="'.trailingslashit(get_option('siteurl')).PLUGINDIR.'/category-icons/images/spacer.gif" />';
						$img .= $spacer_left_image.$image.$spacer_right_image;
					}
					else {
						$img .= $image;
					}
					$i++;
				}
				if ( isset($p) && 1 == $p['icons_only'] && !empty($img) ) {
					$list = str_replace($child[2],'>'.$img.'<', $list);
				}
				else {// Inject icon into the list
					$before_name = bm_caticons_getbool(get_option('igcaticons_before_name'));
					if ($before_name) { // put the html code of the icons before the category name
						$list = str_replace($child[0],$img.$child[0], $list); 
					} else {// else after the category name
						$list = str_replace($child[0], $child[0].$img, $list);
					}
				}
			}
		}
		echo $list;
	}
	else {
		_e("put_cat_icons : your list is empty ! Don't forget to add <strong>echo=0</strong> in the parameters of wp_list_categories().",'category_icons');
	}
}
/**
 * Localization of the plugin description
 * @param string The string to process
 * @return string The string to display
 */
function bm_caticons_plugin_description( $string) {
	if (trim($string) == 'Assign icons to your categories easily. Thanks to the following contributors : Kristian Bollnow, Hugo Chen, Kalin Dimitrov, Gianni Diurno, Samuel Kroslak, Jean-Christophe Marie, Andrew Senyshyn, Henrik Schack, Vincent Sparreboom, TechnopodMan, Oliver Weichhold.')
		$string = __('<a href="edit.php?page=category_icons.php&#38;ig_module=caticons&#38;ig_tab=options">Assign icons to your categories easily</a>. Don\'t forget to activate the Category Icons widget !','category_icons');
	return $string;
}

/**
 * Get the current user's role
 * @return string User role 
 */
function bm_caticons_getuserrole() {
	global $current_user, $wp_roles;
	$user_role = '';
	if( $current_user->id )  
		foreach($wp_roles->role_names as $role => $Role) 
			if ( array_key_exists($role, $current_user->caps) )
				$user_role = $role;
	return $user_role;
}

/**
 * Displays informations if there's an update of the plugin (if check for update is checked)
 */
function bm_caticons_update_nag() {
	$infos = bm_plugin_infos();
	$plugins = get_plugins();
	$msg = '';
	$current_version = trim($plugins['category-icons/category_icons.php']['Version']);
	$plugin_title = $plugins['category-icons/category_icons.php']['Title'];
	if ( $infos && (strval($current_version) < strval($infos['version'])) ) {
		$msg .= sprintf(__('A new version of the plugin <strong>%s</strong> is available','category_icons'),$plugin_title);
		$msg .= ' ('.'<a target="_blank" href="http://categoryicons.wordpress.com/category/changelog/">';
		$msg .= sprintf(__('See the changelog of %s version','category_icons'),$infos['version']);
		$msg .= '</a>)';
	}
	//if (!empty($msg)) echo "<div id='update-nag'>$msg</div>";
	if (!empty($msg)) echo "<div class='updated'><p>$msg</p></div>";
}

/**
 * Displays informations in the footer of WordPress 2.5
 */
function bm_caticons_credits() {
	$translator_name = (__('translator_name','category_icons')) == 'translator_name' ? '' : __('translator_name','category_icons');
	$translator_url  = (__('translator_url','category_icons')) == 'translator_url' ? '' : __('translator_url','category_icons') ;
	$lang = get_locale();
	$plugins = get_plugins();
	$current_version = trim($plugins['category-icons/category_icons.php']['Version']);
	if ('1.9.9' == $current_version ) $current_version .= ' "Twins Edition"';
	$msg = 'Category Icons '.$current_version.'&nbsp;&nbsp;<img src="'.trailingslashit(get_option('siteurl')).PLUGINDIR.'/category-icons/images/w3c-xhtml1.0.png" /> | ';
	$msg .= '<a target="_blank" href="http://categoryicons.wordpress.com/category/documentation/">'.__('Documentation','category_icons').'</a>';
	$msg .= ' | <a target="_blank" href="http://categoryicons.wordpress.com/category/faq/">'.__('FAQ','category_icons').'</a>';
	$msg .= ' | <a target="_blank" href="http://categoryicons.wordpress.com/troubleshooting/">'.__('Troubleshooting','category_icons').'</a>';
	$msg .= ' | <a target="_blank" href="http://categoryicons.wordpress.com/contact/">'.__('Contact the author','category_icons').'</a>' ;
	if (!empty($lang) && !empty($translator_name)) {
		$msg .= (' | ').__('Translated by','category_icons');
		if (!empty($translator_url))
			$msg .= ' <a target="_blank" href="'.$translator_url.'">'.$translator_name.'</a>';
		else
			$msg .= ' '.$translator_name;
	}
	echo $msg.' <br/>';
}

/**
 * Get some infos about the plugin from WordPress.org : the version, the last time it was updated, and the number of downloads
 * @return array return(version,updated,downloads) or false if no information was found
 */
function bm_plugin_infos() {
	if ( !get_option('igcaticons_checkupdates') ) return false;
	$user = 'submarine';
	$uri = 'http://wordpress.org/extend/plugins/profile/'.$user;
	require_once (ABSPATH . WPINC . '/class-snoopy.php');
	$snoopy = new Snoopy();
	@$snoopy->fetch($uri);
	$datas = $snoopy->results;
	$results = array();
	if (!empty($datas) && preg_match_all('#<li><span[^>]*>(\n.*|.*)</span>(.*)</li>#Ui', $datas, $correspondances, PREG_SET_ORDER)) 
		foreach ($correspondances as $correspondance) 
			if ( isset($correspondance[1] ) && isset($correspondance[2])) 
				$results[trim( strtolower($correspondance[1]) )] = trim($correspondance[2]);
	
	if ( 0 < count($results) ) 
		return $results;
	else 
		return false;
}

/**
 * Return the boolean corresponding to $var
 * @param boolean|int|string $var boolean or string or int to eval
 * @return boolean
 */
function bm_caticons_getbool($var) {// i made this function because casting get_option() didn't react as I wanted
	if (is_bool($var)) return $var;
    switch (strtolower($var)) {
        case ('true'): 
			return true;
        case ('false'): 
        default: 
			return false;
    }
}

/**
 * Extract the urls & labels
 * @param string $string String containing the urls to extract
 * @return array
 */
function bm_caticons_url_extractor($string) {
	$myarray = array();
	if (preg_match_all('/(<a\s+.*?href=[\"\']?([^\"\' >]*)[\"\']?[^>]*(>.*?<)\/a>)/i', $string, $correspondances, PREG_SET_ORDER)) 
		foreach ($correspondances as $correspondance) 
			array_push($myarray, array($correspondance[1], $correspondance[2], $correspondance[3]));
	return $myarray;
}

/**
 * Find where to place the code (get_cat_icons()) in the template files. This function'll work for numerous templates.
 * For the majority of the templates I've seen, the post title is after an anchor tag or after a h2 tag.
 * So the function search for : <a href...>the_title()</a> and <h2...>the_title()</h2>.
 * @param string $filename Name of the template folder
 * @return array
 */
function bm_caticons_template_code($filename='') {
	$files = array();
	$results = array();
	$correspondances = array();
	$filename='';
	$notfound = 0;
	$path = ABSPATH.'wp-content/themes/'.get_option('template');
	$getcat_code = "\n<?php if (function_exists('get_cat_icon')) get_cat_icon(); ?>";
	
	if ($handle = opendir($path)) {
		while (false !== ($filename = readdir($handle))) 
			if ( '.php' == trim(substr($filename, strrpos($filename, '.'), strlen($filename)-strrpos($filename, '.'))) ) 
				$files[] = trim($filename);
		closedir($handle);
	}

	foreach ($files as $filename) {// Scan all the php files found for the linked (a href) the_title() function
		$fullfilename = $path.'/'.$filename;
		$string = file_get_contents($fullfilename);
		$found = 0;
		
		if ( (false === strpos(strtolower($string),'put_cat_icons(')) ) {
			if (preg_match_all('/(wp_list_cat?[^\)].*\)[;\s])/i', $string, $correspondances, PREG_SET_ORDER)) {
				$result = array();
				foreach ($correspondances as $correspondance) {
					if ( isset($correspondance[1]) ) 
						$result[] = $correspondance[1];
				}
				
				$correspondances = array_unique($result);
				foreach ($correspondances as $correspondance) {
					if (strpos($correspondance,'(')>0) {// Extract the wp_list_categories() parameters
						$parameters = '';
						if (preg_match_all('/[\(](.*[^\)]\))/i', $correspondance, $trouves, PREG_SET_ORDER)) {
							$parameters = substr($trouves[0][1],0,strlen($trouves[0][0])-2);
							if ( false === !strpos($parameters,'(') ) 
								$parameters = $trouves[0][1];
						}
						
						// Put the code
						$putcat_code = "\nif (function_exists('put_cat_icons')) \n\tput_cat_icons( wp_list_categories(".$parameters;
						if (!empty($parameters)) 
							$putcat_code .=".'&";
						else
							$putcat_code .= '\'';	
						$putcat_code .= "echo=0'));\nelse\n\twp_list_categories(".$parameters.')';
						if (strrpos($correspondance,';')>0)
							$putcat_code .= ';';
						if ( 1 == get_option('igcaticons_templatecode_patch') && is_writable($path.'/'.$filename)) {// patch the files
							$handle = fopen($path.'/'.$filename, "wb");
							$string = str_replace($correspondance,$putcat_code,$string);
							if (fwrite($handle, $string)) $results = array($filename,'-','<i>Ok</i>');
							fclose($handle);
						}
						else {// display the informations about the line and column
							do {
								$position = strpos($string,$correspondance,$position);	
								if ( !( false === $position )	) {					
									$line_number = substr_count(  substr($string, 0,$position)   ,"\n")+1;								
									$column = $position - strrpos (substr($string, 0,$position),"\n") - 1;
									$results[] = array($putcat_code,$filename,$line_number.':'.$column,htmlentities($correspondance));
									$position += strlen($correspondance);
								}
							}
							while ( !( false === $position ));
						}
					}
				}
				
			}
		}
		
		if ( false === strpos(strtolower($string),'get_cat_icon(') ) {
			if (preg_match_all('/(<a\s+.*?href=[\"\']?[^\"\' >]*[\"\']?[^>].*>+[\s\t\r\n]*<\?php +[\/\s\t\r\n$]*?[^>]+[\s\t\r\n]*.*\?>+[\s\t\r\n]*<\/a>)/i', $string, $correspondances, PREG_SET_ORDER)) {
				$correspondances = array_unique($correspondances);
				foreach ($correspondances as $correspondance) {
					if (strpos(strtolower($correspondance[1]),'the_title()')>0) {
						$found = 1;
						if (get_option('igcaticons_templatecode_patch')==1 && is_writable($path.'/'.$filename)) {// patch the files
							$handle = fopen($path.'/'.$filename, "wb");
							$string = str_replace($correspondance[1],$getcat_code.$correspondance[1],$string);
							if (fwrite($handle, $string)) 
								$results = array($getcat_code,$filename,'-','<i>Ok</i>');
							fclose($handle);
						}
						else {// display the informations about the line and column
							do {
								$position = strpos($string,$correspondance[1],$position);	
								if ( !( false === $position )	) {						
									$line_number = substr_count(  substr($string, 0,$position)   ,"\n")+1;								
									$column = $position - strrpos (substr($string, 0,$position),"\n") - 1;
									$results[] = array($getcat_code,$filename,$line_number.':'.$column,htmlentities($correspondance[1]));
									$position += strlen($correspondance[1]);
								}
							}
							while  (!( false === $position ));
						}
					}
				}
			}
			if ( 0 == $found && preg_match_all('#<h2[^>]*>+[\s\t\r\n]*(<\?php +[\/\s\t\r\n$]*?[^>]+[\s\t\r\n]*.*\?>+[\s\t\r\n]*</h2>)#Ui', $string, $correspondances, PREG_SET_ORDER)) {
				$correspondances = array_unique($correspondances);
				foreach ($correspondances as $correspondance) {
					if (strpos(strtolower($correspondance[1]),'the_title()')>0) {
						if ( 1 == get_option('igcaticons_templatecode_patch') && is_writable($path.'/'.$filename)) {// patch the files
							$handle = fopen($path.'/'.$filename, "wb");
							$string = str_replace($correspondance[1],$getcat_code.$correspondance[1],$string);
							if (fwrite($handle, $string)) $results = array($getcat_code,$filename,'-','<i>Ok</i>');
							fclose($handle);
						}
						else {// display the informations about the line and column
							do {
								$position = strpos($string,$correspondance[1],$position);	
								if ( !( false === $position )	) {							
									$line_number = substr_count(  substr($string, 0,$position)   ,"\n")+1;								
									$column = $position - strrpos (substr($string, 0,$position),"\n") - 1;
									$results[] = array($getcat_code,$filename,$line_number.':'.$column,htmlentities($correspondance[1]));
									$position += strlen($correspondance[1]);
								}
							}
							while (!( false === $position ));
						}
					}
				}
			}	
		}
		
	}
	return $results;
}

/**
 * Display code template table panel.
 * @param string $message String to display before the table
 * @param string $code String representing the code to paste
 * @param array $list Every row to display
 * @param string $filter Which type of table to display
 */
function bm_caticons_codetemplate_display($message,$code,$list,$filter) {
	$table = array();
	foreach ($list as $element) {
		if (strpos($element[0],$filter)) $table[] = $element;
	}
	$list = $table;
	if ( 0 < count($list) ) {
?>
         <legend><?php echo '<p>'.$message.'</p>'; ?></legend>
           <?php if ( 'get_cat_icon' == $filter ) echo '<p><pre>'.htmlentities($list[0][0]).'</pre></p>'; ?>
            <table class="widefat">
            	<thead>
                <tr>
                    <th ><?php _e('Filename','category_icons'); ?></td>
                    <th style="text-align:center"><?php _e('Line Number : Column','category_icons'); ?></td>
                    <th style="text-align:center"><?php _e('Code','category_icons'); ?></td>
                <tr>
                </thead>
                <tbody>
                <?php 
                    if ( 0 == count($list) ) 
						$list = array(array('-','-','-'));
                    foreach ($list as $element) {	
                        $class = ('alternate' == $class) ? '' : 'alternate';
                        ?>
                    <tr class="<?php echo $class; ?>">
                        <th><?php echo $element[1]; ?></th>
                        <td align="center"><?php echo $element[2]; ?></td>
                        <td align="center"><?php 
							if ( 'get_cat_icon' == $filter ) 
								echo '<img src="'.trailingslashit(get_option('siteurl')).PLUGINDIR.'/category-icons/images/arrowdown.png" />';
							echo $element[3];
							if ( 'put_cat_icon' == $filter )
								echo '<br/><b>'.__('by','category_icons').'</b><br/>'.htmlentities($element[0]).'<br/>';
							?></td>
                    </tr>
                        <?php
                    }
                ?></tbody>
         </table>
  <?php
     }
}

/**
 * Check if the default upload path is readable.
 */
function bm_caticons_check_uploads_path() {
	if (!is_readable(ig_caticons_path())) {
	?>
        <div id="message" class="updated fade">
            <p><strong>
        <?php 
            echo __('Error in Category Icons','category_icons').' : '.__('the default upload path is not accessible. Please create one or change the permissions on the directory.','category_icons'); 
        ?>
            </strong></p>
        </div>
    <?php
	}
}

/**
 * Echoes the option tag filled with icons paths
 * @param string $selected the icon to select in the dropdown menu
 */
function bm_caticons_get_icons_paths($selected='') {
	$files = array();
	$basepath = ig_caticons_path();
	$files = bm_caticons_recursive_readdir($basepath);
	foreach ($files as $file) {
		$sel = ($selected==$file) ? ' selected="selected"' : '';
		$entry_escaped = htmlspecialchars($file);
		echo("<option value=\"$entry_escaped\"$sel>$entry_escaped</option>");
	}
}

/**
 * Gets the icons paths
 * @param string $folder Name of the folder to browse
 * @param array $files Filenames already found
 * @return array
 */
function bm_caticons_recursive_readdir($folder,$files=array()) {
	$separator = '/';
	$types = explode(' ',get_option('igcaticons_filetypes'));
	if (is_readable($folder) && $handle = opendir($folder)  ) {
		while (false !== ($filename = readdir($handle))) {
			if ($filename !== '.' && $filename !== '..') {
				$path = $folder.$separator.$filename;
				if (is_dir ($path)) 
					$files = array_unique(array_merge($files, bm_caticons_recursive_readdir($path,$files))); // recursive call
				else {// files
					$p = strrpos($filename,'.');
					if (false===$p || !in_array(strtolower(substr($filename, $p+1)), $types)) continue;
					else
						$files[] = str_replace(ig_caticons_path().$separator,'', trim($path));
				}
			}
		}
		closedir($handle);
	}
	return $files;
}

/**
 * Print category rows for admin panel for WordPress 2.5
 * @param int $parent Category ID of the parent
 * @param int $level Level of the category (parent or child)
 * @param array $category Array of categories (Objects)
 * @return false False if no category is found
 */
function bm_caticons_rows( $parent = 0, $level = 0, $categories = 0 ) { // I took the code from template.php of WordPress 2.5 and modified it a little
	if ( !$categories ) {
		$args = array('hide_empty' => 0);
		if ( !empty($_GET['s']) )
			$args['search'] = $_GET['s'];
		$categories = get_categories( $args );
	}

	$children = _get_term_hierarchy('category');

	if ( $categories ) {
		ob_start();
		foreach ( $categories as $category ) {
			if ( $category->parent == $parent) {
				echo "\t" . _bm_caticons_row( $category, $level );
				if ( isset($children[$category->term_id]) )
					bm_caticons_rows( $category->term_id, $level +1, $categories );
			}
		}
		$output = ob_get_contents();
		ob_end_clean();

		$output = apply_filters('bm_caticons_rows', $output);

		echo $output;
	}
	else
		return false;
}

/**
 * @return string 
 */
function _bm_caticons_row( $category, $level/*, $name_override = false*/ ) { // I took the code from template.php of WordPress 2.5 and modified it a little
	global $class;
	$icons_path = ig_caticons_path();
	$category = get_category( $category );
	$role = bm_caticons_getuserrole();
	$pad = str_repeat( '&#8212; ', $level );
	//$name = ( $name_override ? $name_override : $pad . ' ' . $category->name );
	$name =  $pad . ' ' . $category->name ;
	list($priority, $icon, $small_icon) = ig_caticons_get_icons($category->term_id);
	
	$icon_cell = '';
	if ( !empty($icon) && is_readable( $icons_path.'/'.$icon ) ) {
		list($width, $height, $type, $attr) = getimagesize($icons_path.'/'.$icon);
		list($w, $h) = ig_fit_rect($width, $height, 100, 100);
		$icon_cell = '<img src="'.ig_caticons_url()."/$icon\" width=\"$w\" height=\"$h\" /> <br />$icon ($width x $height)";
	}
	
	$small_icon_cell = '';
	if ( !empty($small_icon) && is_readable( $icons_path.'/'.$small_icon ) ) {
		list($width, $height, $type, $attr) = getimagesize($icons_path.'/'.$small_icon);
		list($w, $h) = ig_fit_rect($width, $height, 100, 100);
		$small_icon_cell = '<img src="'.ig_caticons_url()."/$small_icon\" width=\"$w\" height=\"$h\" /> <br />$small_icon ($width x $height)";
	}
	
	$edit = $name;
	if ('administrator' == $role || 'editor' == $role) 
		$edit = "<a class='row-title' href='".IG_ADMIN_SELF."&ig_module=caticons&ig_tab=icons&action=edit&cat_ID=$category->term_id' title='"
				. attribute_escape(sprintf(__('Edit %s','category_icons'), $category->name)) . "'>" .$name . "</a>";

	$class = " class='alternate'" == $class ? '' : " class='alternate'";

	$category->count = number_format_i18n( $category->count );
	$posts_count = $category->count;
	
	// Prepare the output string
	$output = '<tr id=cat-"'.$category->term_id.'" '.$class.' >'.
			   '<th scope="row" style="text-align:center;vertical-align:middle;padding:7px 0 8px">';
	if ('administrator' == $role || 'editor' == $role) 
		$output .= '<input type="checkbox" name="delete[]" value="'.$category->term_id.'" /></th>';
	else 
		$output .= "&nbsp;";
	
	$output .=  '<td style="vertical-align:middle;">'.$category->term_id.'</td>'.
				'<td style="vertical-align:middle;">'.$edit.'</td>'.
				'<td class="num" style="vertical-align:middle;text-align:center" align="center" >'.$posts_count.'</td>'.
				'<td class="num" style="vertical-align:middle;text-align:center" align="center">'.$priority.'</td>'.
				'<td style="vertical-align:middle;text-align:center" align="center">'.$icon_cell.'</td>'.
				'<td style="vertical-align:middle;text-align:center" align="center">'.$small_icon_cell.'</td>'.
				"\n\t</tr>\n";

	return apply_filters('bm_caticons_row', $output);
}

/**
 * Loads forms.js (which is loaded as 'admin-forms')
 * I need it because of the multiple checks in one click
 */
function bm_caticons_add_js_libs() {
	if ( strval($GLOBALS['wp_version']) < strval('2.5') )
		wp_enqueue_script('caticons-delete', '/wp-content/plugins/category-icons/js/category_icons.js');
	else
		wp_enqueue_script('admin-forms');//WP 2.5+ only
	wp_enqueue_script('jquery');
	wp_enqueue_script('caticons-panel', '/wp-content/plugins/category-icons/js/category_icons-panel.js');
}

function bm_caticons_adminicons() { // I took some of the code from categories.php of WordPress 2.5 and modified it a little
	global $wpdb;
	$role = bm_caticons_getuserrole();
	$action = isset($_REQUEST['action']) ? $_REQUEST['action'] : '';
	
	if ( isset($_GET['deleteit']) && isset($_GET['delete']) )
		$action = 'bulk-delete';
	
	switch($action) {
		case 'update-category-icon':
			$cat_ID = (int) $_GET['cat_ID'];
			$priority = $_REQUEST['ig_priority'];
			$icon = $_REQUEST['ig_icon'];
			$small_icon = $_REQUEST['ig_small_icon'];
			if ($wpdb->get_var($wpdb->prepare("SELECT cat_id FROM $wpdb->ig_caticons WHERE cat_id='$cat_ID'"))) {
				$wpdb->query($wpdb->prepare("UPDATE $wpdb->ig_caticons SET priority='$priority', icon='$icon', small_icon='$small_icon' WHERE cat_id='$cat_ID'"));
			} else {
				$wpdb->query($wpdb->prepare("INSERT INTO $wpdb->ig_caticons (cat_id, priority, icon, small_icon) VALUES ('$cat_ID', '$priority', '$icon', '$small_icon')"));
			}
			
		break;
		
		case 'delete':
			$cat_ID = (int) $_GET['cat_ID'];
			
			if ( 'editor' != $role && 'administrator' != $role )
				wp_die(__('Are you trying to cheat ?','category_icons'));
				
			$cat_name = get_catname($cat_ID);
			$request = "DELETE FROM $wpdb->ig_caticons WHERE cat_id='$cat_ID'";
			if (false === $wpdb->query($wpdb->prepare($request) ))
				//wp_redirect('category-icons.php?message=3');
				wp_die(__('Error in Category Icons','category_icons').' : '.$request);
			
		break;
		
		case 'bulk-delete':
			if ( 'editor' != $role && 'administrator' != $role )
				wp_die( __('You are not allowed to delete category icons.','category_icons') );
		
			foreach ( (array) $_GET['delete'] as $cat_ID ) {
				$cat_name = get_catname($cat_ID);
				$wpdb->query($wpdb->prepare("DELETE FROM $wpdb->ig_caticons WHERE cat_id='$cat_ID'"));
			}
		break;
	}
	
	switch ($action) {
	case 'edit':
		$cat_ID = (int) $_GET['cat_ID'];
		$category = get_category_to_edit($cat_ID);
		list($priority, $icon, $small_icon) = ig_caticons_get_icons($cat_ID);
		?>
		<div class="wrap">
		<h2><?php _e('Select Category Icons','category_icons') ?> </h2>
		<form method="post" action="">
			<input type="hidden" name="ig_module" value="caticons" />
			<input type="hidden" name="ig_tab" value="icons" />
			<input type="hidden" name="action" value="update-category-icon" />
            <table cellspacing="1" cellpadding="2" border="0" class="form-table">
                <tr>
                    <th scope="row" style="vertical-align:text-top;"><?php _e('Category ID','category_icons'); ?></th>
                    <td colspan="2" ><?php echo $cat_ID;?></td>
                </tr>
                <tr>
                    <th scope="row" style="vertical-align:text-top;"><?php _e('Name','category_icons'); ?></th>
                    <td colspan="2"><?php echo $category->name;?></td>
                </tr>
                <tr>
                    <th scope="row" class="num" style="vertical-align:text-top;"><?php _e('Priority','category_icons'); ?></th>
                    <td colspan="2">
                        <input type="text" name="ig_priority" size="5" value="<?php echo $priority; ?>" />
                    </td>
                </tr>
                <tr>
                    <th scope="row" style="vertical-align:text-top;"><?php _e('Icon','category_icons'); ?></th>
                    <td valign="top">
                        <select name="ig_icon" rows="10" onChange="icon_preview.src=('<?php echo ig_caticons_url();?>/'+this.options[this.selectedIndex].value);">
                            <option value="">--- <?php _e('No Icon','category_icons'); ?> ---</option>
                            <?php bm_caticons_get_icons_paths($icon);	?>
                        </select>
                    </td>
                    <td valign="top"><img id="icon_preview" src="<?php echo ig_caticons_url()."/$icon";?>" /></td>
                </tr>
                <tr>
                    <th scope="row" style="vertical-align:text-top;"><?php _e('Small Icon','category_icons'); ?></th>
                    <td valign="top">
                        <select name="ig_small_icon" rows="10" onChange="small_icon_preview.src=('<?php echo ig_caticons_url();?>/'+this.options[this.selectedIndex].value);">
                            <option value="">--- <?php _e('No Icon','category_icons'); ?> ---</option>
                            <?php bm_caticons_get_icons_paths($small_icon); ?>
                        </select>
                    </td>
                    <td valign="top"><img id="small_icon_preview" src="<?php echo ig_caticons_url()."/$small_icon";?>" /></td>
                </tr>
            </table>
			<div class="submit"><input type="submit" name="info_update" value="<?php _e('Select Icons','category_icons');?> &raquo;" /></div>
        </form>
		</div>
		<?php
	break;
	
	default:
	?>
	<div class="wrap">
		<!--<form id="posts-filter" action="" onsubmit="return checkThis(this)" method="get">-->
        <form id="posts-filter" action="" method="get">
            <input type="hidden" name="ig_module" value="caticons" />
            <input type="hidden" name="page" value="category_icons.php" />
            <input type="hidden" name="action" value="delete" />
            <input type="hidden" name="ig_tab" value="icons" />
		<h2><?php _e('Select Category Icons','category_icons') ?> </h2>
		<div class="tablenav">
			<div class="alignleft">
				<input type="submit" value="<?php _e('Delete icons and priority','category_icons'); ?>" name="deleteit" class="button-secondary delete" />
			</div>
			<br class="clear" />
		</div>
		<br class="clear" />
		<table class="widefat">
			<thead>
			<tr>
				<th scope="col" style="text-align:center;vertical-align:middle;padding:8px 0 8px"><input type="checkbox" onclick="checkAll(document.getElementById('posts-filter'));" /></th>
				<th scope="col"><?php _e('ID','category_icons') ?></th>
                <th scope="col"><?php _e('Name','category_icons') ?></th>
                <th scope="col" class="num" style="text-align:center"><?php _e('Posts','category_icons') ?></th>
                <th scope="col" style="text-align:center"><?php _e('Priority','category_icons') ?></th>
				<th scope="col" style="text-align:center"><?php _e('Icon','category_icons') ?></th>
				<th scope="col" style="text-align:center"><?php _e('Small Icon','category_icons') ?></th>
			</tr>
			</thead>
			<tbody id="the-list" class="list:cat">
		<?php
		bm_caticons_rows();
		?>
			</tbody>
		</table>
		</form>
        <div class="tablenav">
        <br class="clear" />
        </div>
        <br class="clear" />
        </div>
	<?php
	}// end switch
}

/**
 * If the '$field' field doesn't exist in the table, create it if query is provided, else throw an error.
 * @param string $field table field 
 * @param string $query SQL query
 */
function bm_caticons_checkdbfield($field,$query='') { 
	global $wpdb;
	$fieldb = $wpdb->get_results($wpdb->prepare("show columns from `$wpdb->ig_caticons`"),ARRAY_A);
	$found = false;
	if ( 0 < count($fieldb) ) 
		foreach ($fieldb as $array) {// Search for the field $field
			if ( $array['Field'] == $field ) 
				$found = true;
		}
	
	if ( !$found ) {// if the field is not found, 
		if (!empty($query) ) {// if the query is not empty
			$wpdb->query($wpdb->prepare($query)); // execute it
		}
		else {// If it is found, but no query is provided, we throw an error ?>
            <div id="message" class="updated fade"><p><strong>
            <?php 
                echo __('Error in Category Icons','category_icons').' : '.__('you must deactivate and reactivate this plugin.','category_icons'); 
                echo ' ';
				printf(__('The field %s is missing.','category_icons'),$field);
            ?>
            </strong></p></div>
    <?php		exit();
		}
	}
}

/**
 * Install the plugin
 */
function ig_caticons_install() {
	global $wpdb, $table_prefix;
	$wpdb->query($wpdb->prepare("CREATE TABLE IF NOT EXISTS `$wpdb->ig_caticons` (
					`cat_id` INT NOT NULL ,
					`priority` INT NOT NULL ,
					`icon` TEXT NOT NULL ,
					`small_icon` TEXT NOT NULL ,
					PRIMARY KEY ( `cat_id` ))"));
	// Check if priority field exists and create it if necessary					
	bm_caticons_checkdbfield('priority',$wpdb->prepare("ALTER TABLE `$wpdb->ig_caticons` ADD `priority` INT NOT NULL ;"));
	add_option('igcaticons_path', '');
	add_option('igcaticons_url', '');
	add_option('igcaticons_filetypes', 'jpg gif jpeg png');
	add_option('igcaticons_max_icons','3');
	add_option('igcaticons_before_name','true');
	add_option('igcaticons_fit_width','-1');
	add_option('igcaticons_fit_height','-1');
	add_option('igcaticons_use_small','true');
	add_option('igcaticons_templatecode_patch','0');
	add_option('igcaticons_templatecode_sidebar','1');
	add_option('igcaticons_sidebar_leftspacer_width', '0');
	add_option('igcaticons_sidebar_leftspacer_height', '0');
	add_option('igcaticons_sidebar_rightspacer_width', '0');
	add_option('igcaticons_sidebar_rightspacer_height', '0');
	add_option('igcaticons_checkupdates','1');
	add_option('igcaticons_rssfeeds','1');
	add_option('igcaticons_spacerstype','image');
	
}

/**
 * Get the upload & siteurl paths
 * @return array (path, url) 
 */
function ig_caticons_defupload() {
	$def_path = str_replace(ABSPATH, '', get_option('upload_path')); // wordpress's option
	$def_url = trailingslashit(get_option('siteurl')) . $def_path; // idem
	return array($def_path, $def_url);
}

/**
 * Integrate into the admin menu.
 */
function ig_caticons_adminmenu() {
	load_plugin_textdomain('category_icons','wp-content/plugins/category-icons/languages/');
	define('IG_ADMIN_SELF', $_SERVER['SCRIPT_URL'].'?page='.$_REQUEST['page']);
	
	if (function_exists('add_submenu_page'))
		add_submenu_page('edit.php', __('Category Icons','category_icons') , __('Category Icons','category_icons') , 'manage_categories' , basename(__FILE__) , 'ig_caticons_adminpanel' );	
}

/**
 * Handle admin panel
 */
function ig_caticons_adminpanel() {
	if(isset($_REQUEST['_wpnonce']) ) {
			if(function_exists('check_admin_referer')) {
				check_admin_referer('wordpress_automatic_upgrade');
			}
		}
		
	add_action( 'in_admin_footer', 'bm_caticons_credits' );
		
?>
<style type='text/css' media='all'>
#caticonsmenu {
	margin: 0;
	list-style: none;
	padding: <?php echo (strval($GLOBALS['wp_version']) < strval('2.5')) ? '15px 0 0px 25px' : '7px 0 15px 0'; ?> ;
}

#caticonsmenu li {
	display: inline;
	line-height: 200%;
	list-style: none;
	text-align: center;
	white-space: nowrap;
	margin: 0;
	padding: 0 0 0px 17px;
}

#caticonsmenu ul {
	margin: 0;
	position: absolute;
	margin-left: -0.2em;
}

#caticonsmenu a:hover {
	color: #006699;
}


#caticonsmenu a.current {
	color: #006699; 
}

#caticonspanel {
	display: none;
}
</style>
<?php
	if (!(file_exists( ABSPATH.'/'.PLUGINDIR.'/category-icons/category_icons.php'))) {
?>
		<div id="message" class="updated fade">
        	<p><strong>
		<?php 
			echo __('Error in Category Icons','category_icons').' : '.__('you must put the category_icons.php in /wp-content/plugins/category-icons/ directory.<br/> So deactivate the plugin, and move the file into category-icons directory.<br/>Click on Plugins 2 times, to restart the plugin detection.<br/>And then, reactivate it.','category_icons'); 
		?>
        	</strong></p>
        </div>
<?php
	}
	bm_caticons_check_uploads_path();
	$tab = isset($_REQUEST['ig_tab']) ? $_REQUEST['ig_tab'] : '';
?>
	<ul id="<?php echo (strval($GLOBALS['wp_version']) < strval('2.5')) ? 'caticonsmenu' : 'submenu'; ?>">
       <li style="font-size:12px"> <a href="<?php echo IG_ADMIN_SELF;?>&ig_module=caticons&ig_tab=icons" <?php if ($tab=='icons') echo 'class="current"'; ?>>
	   		<?php _e('Icons','category_icons'); ?></a> </li>
       <li style="font-size:12px"> <a href="<?php echo IG_ADMIN_SELF;?>&ig_module=caticons&ig_tab=options" <?php if ($tab=='options') echo 'class="current"'; ?>>
	   		<?php _e('Options','category_icons'); ?></a> </li>
      <li style="font-size:12px">  <a href="<?php echo IG_ADMIN_SELF;?>&ig_module=caticons&ig_tab=template" <?php if ($tab=='template') echo 'class="current"'; ?>>
	  		<?php _e('Template Code','category_icons'); ?></a></li>
	 </ul> 
<?php
	
	switch ($tab) {
		case ('icons'):
			bm_caticons_adminicons();
		break;
		case ('template'):
			ig_caticons_admintemplate();
		break;
		case ('options'):
		default:
			ig_caticons_adminoptions();
	}
}

/**
 * Handle options tab (plugin section) from the admin panel.
 */
function ig_caticons_adminoptions() {
	 
	if (isset($_POST['info_update'])) {
		?><div class="updated"><p><strong><?php _e('Settings updated.','category_icons');?></strong></p></div><?php
		update_option('igcaticons_path', $_POST['igcaticons_path']);
		update_option('igcaticons_url', $_POST['igcaticons_url']);
		update_option('igcaticons_filetypes', $_POST['igcaticons_filetypes']);
		update_option('igcaticons_max_icons', $_POST['igcaticons_max_icons']);
		update_option('igcaticons_before_name', $_POST['igcaticons_before_name']);
		update_option('igcaticons_templatecode_patch', $_POST['igcaticons_templatecode_patch']);
		update_option('igcaticons_templatecode_sidebar', $_POST['igcaticons_templatecode_sidebar']);
		update_option('igcaticons_sidebar_leftspacer_width', $_POST['igcaticons_sidebar_leftspacer_width']);
		update_option('igcaticons_sidebar_leftspacer_height', $_POST['igcaticons_sidebar_leftspacer_height']);
		update_option('igcaticons_sidebar_rightspacer_width', $_POST['igcaticons_sidebar_rightspacer_width']);
		update_option('igcaticons_sidebar_rightspacer_height', $_POST['igcaticons_sidebar_rightspacer_height']);
		update_option('igcaticons_checkupdates', $_POST['igcaticons_checkupdates']);
		update_option('igcaticons_rssfeeds', $_POST['igcaticons_rssfeeds']);
		update_option('igcaticons_spacerstype', $_POST['igcaticons_spacerstype']);
		
	}
	list($def_path, $def_url) = ig_caticons_defupload();
?>
<div class="wrap">
  <form method="post" action="">
	<input type="hidden" name="ig_module" value="caticons">
	<input type="hidden" name="ig_tab" value="options">
    <h2><?php _e('Category Icons Settings','category_icons');?></h2>
		<table <?php echo (strval($GLOBALS['wp_version']) < strval('2.5')) ? 'class="optiontable"' : 'class="form-table"'; ?> >
<tr valign="top">
				<th ><?php _e('Local Path to Icons','category_icons');?></th>
	    <td ><input type="text" name="igcaticons_path" size="50" value="<?php echo htmlspecialchars(get_option('igcaticons_path')); ?>" /><br /><?php _e('Leave blank to use default upload path','category_icons');?> (<?php echo htmlspecialchars($def_path); ?>).<br/><br/></td>
		  </tr>
			<tr valign="top">
				<th ><?php _e('URL to Icons','category_icons');?></th>
			  <td ><input type="text" name="igcaticons_url" size="50" value="<?php echo htmlspecialchars(get_option('igcaticons_url')); ?>" /><br /><?php _e('Leave blank to use default upload url','category_icons');?> (<?php echo htmlspecialchars($def_url); ?>).<br/><br/></td>
		  </tr>
			<tr valign="top">
				<th ><?php _e('Image File Types','category_icons');?></th>
			  <td><input type="text" name="igcaticons_filetypes" size="50" value="<?php echo htmlspecialchars(get_option('igcaticons_filetypes')); ?>" /><br /><?php _e("Separate by space or comma. E.g. 'jpg gif jpeg png'",'category_icons');?><br/><br/></td>
		  </tr>
          <tr valign="top">
				<th ><?php _e('Maximum number of icons','category_icons');?></th>
			  <td><input type="text" name="igcaticons_max_icons" size="3" value="<?php echo htmlspecialchars(get_option('igcaticons_max_icons')); ?>" /><br /><?php _e('Enter the maximum number of icons to display in front of the posts titles.','category_icons');?><br/><br/></td>
		  </tr>
          <tr valign="top">
				<th ><?php _e('Template Code','category_icons'); ?></th>
			  <td><label><?php _e('Patch the files if possible (check for true)','category_icons');  ?>
  <input type="checkbox" name="igcaticons_templatecode_patch" id="templatecode_patch" value="1" <?php checked('1', get_option('igcaticons_templatecode_patch')); ?> /><br/><?php _e('Process the list of categories (usually used in sidebar) ?','category_icons');?> </label> &nbsp;
  <input type="checkbox" name="igcaticons_templatecode_sidebar" id="templatecode_sidebar" value="1" <?php checked('1', get_option('igcaticons_templatecode_sidebar')); ?> /><br/><br/>
              </td>
		  </tr>
          <tr valign="top">
			 <th><?php _e('Position the category icon in sidebar on the left<br/>or on the right of the category name ?','category_icons'); ?></th>
			 <td>
              <label>
  <input type="radio" name="igcaticons_before_name" id="before_name" value="true" <?php  $thebool = bm_caticons_getbool(get_option('igcaticons_before_name')); if ($thebool) echo "checked='checked'"; ?>/><?php _e('Left','category_icons');?> </label>&nbsp;
  <label>
  <input type="radio" name="igcaticons_before_name" id="before_name" value="false" <?php if (!$thebool) echo "checked='checked'"; ?>/><?php _e('Right','category_icons');?> </label>
              <br /><?php _e("Select 'Left' if you want to put the category icon in front of the category name in the sidebar, or 'Right' if you want to put it after. <b>Remember that you must use the put_cat_icons() function to display icons in the sidebar</b>.",'category_icons');?><br/><br/></td>
		  </tr>
          <tr valign="top">
			 <th ><?php _e('Sizes of the margins around the sidebar icons','category_icons'); ?></th>
			 <td align="left">
			 <label><input type="radio" checked="checked" name="igcaticons_spacerstype" value="css" <?php  $thebool = get_option('igcaticons_spacerstype'); if ('css' == $thebool) echo "checked='checked'"; ?>/><?php _e('Use CSS (in this case, you must handle yourself the space around the icons with margin)','category_icons'); ?></label><br/><label><input name="igcaticons_spacerstype" type="radio" id="radioimage" value="image" <?php if ('image' == $thebool) echo "checked='checked'"; ?> /><?php _e('Use an image to create spaces around the icons','category_icons'); ?></label>
             <div id="caticonspanel">
			 <?php _e('Left Spacer Width','category_icons'); ?>
             <input type="text" name="igcaticons_sidebar_leftspacer_width" size="3" value="<?php  echo get_option('igcaticons_sidebar_leftspacer_width'); ?>" />&nbsp;&nbsp;
			 <?php _e('Left Spacer Height','category_icons'); ?>
             <input type="text" name="igcaticons_sidebar_leftspacer_height" size="3" value="<?php  echo get_option('igcaticons_sidebar_leftspacer_height'); ?>" /><br/>
			 <?php _e('Right Spacer Width','category_icons'); ?>
             <input type="text" name="igcaticons_sidebar_rightspacer_width" size="3" value="<?php  echo get_option('igcaticons_sidebar_rightspacer_width'); ?>" />&nbsp;&nbsp;
			 <?php _e('Right Spacer Height','category_icons'); ?><input type="text" name="igcaticons_sidebar_rightspacer_height" size="3" value="<?php  echo get_option('igcaticons_sidebar_rightspacer_height'); ?>" />
             
            </div><br/><br/></td>
		  </tr>
          <tr valign="top"><th ><?php _e('Check for updates','category_icons'); ?></th><td><label>
  <input type="checkbox" name="igcaticons_checkupdates" id="checkupdates" value="1" <?php checked('1', get_option('igcaticons_checkupdates')); ?> /></label><br/><br/></td></tr>
  	<tr valign="top"><th style="WHITE-SPACE: nowrap; vertical-align:top"><?php _e('Display category icons in RSS feeds','category_icons'); ?></th><td><label>
  <input type="checkbox" name="igcaticons_rssfeeds" id="rssfeeds" value="1" <?php checked('1', get_option('igcaticons_rssfeeds')); ?> /></label><br/><br/></td></tr>
	 </table>
	<div class="submit"><input type="submit" name="info_update" value="<?php _e('Update Options','category_icons');?> &raquo;" /></div>	
  </form>
</div>
<?php
}

/**
 * Handle template tab (plugin section) from the admin panel.
 */
function ig_caticons_admintemplate() {
	$list = bm_caticons_template_code();
	$message =__('Paste the following code in the listed files, at the line and column numbers displayed below.','category_icons');
	if ( 0 == count($list) ) 
		$message = __('There\'s nothing to do.','category_icons');
?>
<div class=wrap>
    <h2><?php 
			$ct = current_theme_info();
			_e('Template Code for ','category_icons');
			echo $ct->title;
		?>
    </h2>
    <h3><a target="_blank" href="http://categoryicons.wordpress.com/2008/03/12/function-get_cat_icon/">get_cat_icon()</a></h3><p>
<?php 
	if ( 0 < count($list) ) {	
		bm_caticons_codetemplate_display($message,$code,$list,'get_cat_icon'); 
		if (get_option('igcaticons_templatecode_sidebar')==1) {
			echo '<h2>&nbsp;</h2><h3><a target="_blank" href="http://categoryicons.wordpress.com/2008/03/12/function-put_cat_icon/">put_cat_icons()</a></h3><p>';
			$message = __('Replace the code','category_icons');
			bm_caticons_codetemplate_display($message,$code,$list,'put_cat_icon'); 
			echo '</p>';
		}
	}
?>
    </p>
</div>
<?php
}

/**
 * Get the icons base path.
 * @return array
 */
function ig_caticons_path() {
	$path = get_option('igcaticons_path');
	$def = ig_caticons_defupload();
	if (empty($path))
		return ABSPATH.$def[0];
	else
		return ABSPATH.$path;
}

/**
 * Get the icons base url.
 * @return array
 */
function ig_caticons_url() {
	$url = get_option('igcaticons_url');
	$def = ig_caticons_defupload();
	if (empty($url))
		return $def[1];
	else
		return $url;
}

/**
 * Get file types to show when selecting icons.
 * @return array
 */
function ig_caticons_filetypes() {
	$types = get_option('igcaticons_filetypes');
	if (empty($types))
		return get_option('fileupload_allowedtypes');
	else
		return $types;
}

/**
 * Get category icons
 * @param int $cat Category ID
 * @return array (priority, icon, small_icon)
 */
function ig_caticons_get_icons($cat) {
	global $wpdb;
	bm_caticons_checkdbfield('priority'); // in future version, we'll delete this check : priority feature is introduced in version 1.7
	$cat = $wpdb->escape($cat);
	if ($row = $wpdb->get_row($wpdb->prepare("SELECT priority, icon, small_icon FROM $wpdb->ig_caticons WHERE cat_id='$cat'"))) 
		return array($row->priority, $row->icon, $row->small_icon);
	else 
		return false;
}

/**
 * Get category icon with the hightest priority.
 * @param array $cats Array of Category IDs
 * @return array (cat_id, priority, icon, small_icon)
 * @author Oliver Weichhold
 */
function ow_caticons_get_priority_icon($cats) {
	global $wpdb;
	$instr = '';
	bm_caticons_checkdbfield('priority');// in future version, we'll delete this check : priority feature is introduced in version 1.7
	foreach($cats as $cat)
		$instr .= $wpdb->escape($cat).',';
	
	$instr = preg_replace('/,$/','', $instr); // Remove trailing comma
	if ($row = $wpdb->get_row($wpdb->prepare("SELECT cat_id, priority, icon, small_icon FROM $wpdb->ig_caticons WHERE cat_id IN($instr) ORDER BY priority DESC LIMIT 1")))
		return array($row->cat_id, $row->priority, $row->icon, $row->small_icon);
	else
		return false;
}

/**
 * Utility function to compute a rectangle to fit a given rectangle by maintaining the aspect ratio.
 * @param int $width Width of the rectangle
 * @param int $height Height of the rectangle
 * @param int $max_width Maximum Width of the new rectangle
 * @param int $max_height Maximum Height of the new rectangle
 * @param boolean $expand Expand the rectangle ? Default : false
 * @return array (width, height)
 */
 function ig_fit_rect($width, $height, $max_width=-1, $max_height=-1, $expand=false) {
	$h = $height;
	$w = $width;
	if ($max_width>0 && ($w > $max_width || $expand)) {
		$w = $max_width;
		$h = floor(($w*$height)/$width);
	}
	if ($max_height>0 && $h >$max_height) {
		$h = $max_height;
		$w = floor(($h*$width)/$height);
	}
	return array($w,$h);
}

if (!function_exists('stripslaghes_gpc_arr')) {
	function stripslaghes_gpc_arr(&$arr) {
		if (get_magic_quotes_gpc()) {
			foreach(array_keys($arr) as $k) $arr[$k] = stripslashes($arr[$k]);
		}
	}
}
?>